#!/usr/bin/env python
"""
img2pyqt.py:

    Inlines a list of png images, and embed them in a Python
    module with appropriate code so it can be loaded into
    a PyQt program at runtime.  The benefit is that since
    it is Python source code it can be delivered as a .pyc
    or 'compiled' into the program using freeze, py2exe, etc.

Usage:

    img2pyqt.py [options] image_files.. python_file

Options:

    -a  This flag specifies that the python_file should be appended
        to instead of overwritten (if allowed with -o).

    -u  Don't use compression.  Leaves the data uncompressed.

    -o  Overwrite python_file, if exist

Naming:

    Each filename is stripped from ext and non alphanumeric characters,
    and 3 functions are created: fileData(), fileIcon() and fileIconSet().
    If file extension is not one of (png, xpm, bmp), the last two are
    ommited.

Copyright:

    found in wxPython/tools, which implies creator: Robin Dunn

    2002/07/31:
    adjusted to PyQt by Hans-Peter Jansen <hpj@urpla.net>
"""

import sys, os, getopt, time
import cPickle, cStringIO, zlib


def crunch_data(data, compressed):
    # pickle, crunch and convert it to a form suitable for embedding in code
    data = cPickle.dumps(data)
    if compressed:
        data = zlib.compress(data, 9)
    data = repr(data)

    # This next bit is borrowed from PIL.  It is used to wrap the text intelligently.
    fp = cStringIO.StringIO()
    data = data + " "  # buffer for the +1 test
    c = i = 0
    word = ""
    octdigits = "01234567"
    hexdigits = "0123456789abcdef"
    while i < len(data):
        if data[i] != "\\":
            word = data[i]
            i = i + 1
        else:
            if data[i+1] in octdigits:
                for n in range(2, 5):
                    if data[i+n] not in octdigits:
                        break
                word = data[i:i+n]
                i = i + n
            elif data[i+1] == 'x':
                for n in range(2, 5):
                    if data[i+n] not in hexdigits:
                        break
                word = data[i:i+n]
                i = i + n
            else:
                word = data[i:i+2]
                i = i + 2

        l = len(word)
        if c + l >= 78-1:
            fp.write("\\\n")
            c = 0
        fp.write(word)
        c = c + l

    # return the formatted compressed data
    return fp.getvalue()


def main(args):
    if not args or ("-h" in args):
        print __doc__
        return

    append = 0
    compressed = 1
    overwrite = 0
    imgName = ""

    try:
        opts, fileArgs = getopt.getopt(args, "aou")
    except getopt.GetoptError:
        print __doc__
        return

    for opt, val in opts:
        if opt == "-a":
            append = 1
        elif opt == "-o":
            overwrite = 1
        elif opt == "-u":
            compressed = 0

    if len(fileArgs) < 2:
        print __doc__
        return

    image_files = fileArgs[:-1]
    python_file = fileArgs[-1]

    if not append and not overwrite and os.path.exists(python_file):
        print "Error: not in append mode and %s exists" % python_file
        return

    out = None
    while image_files:
        fname = image_files[0]
        del image_files[0]
        f, ext = os.path.splitext(os.path.basename(fname))
        if not f[0].isalpha():
            print "Error: %s doesn't start with an alphabetic character" % f
            continue
        f = map(None, f)
        i = 0
        for c in f:
            if not c.isalnum():
                print "Warning: character <%c> was removed" % c
                del f[i]
            i += 1
        f = "".join(f)
        if not f:
            print "Error: invalid file: %s" % fname
            continue
        varName = f
        ext = ext.lower()
        if ext in ('.png', '.xpm', '.bmp'):
            datatype = "Icon"
        else:
            datatype = "Data"

        data = crunch_data(open(fname, "r").read(), compressed)

        if not out:
            if append:
                out = open(python_file, "a")
            else:
                out = open(python_file, "w")
        if not out:
            print "Error: cannot write to %s" % python_file

        out.write("#" + "-" * 70 + "\n")
        if not append:
            out.write("# This file was generated %s by %s\n#\n"
                      % (time.asctime(time.localtime()), sys.argv[0]))
            out.write("from qt import QIconSet, QPixmap\n")
            if compressed:
                out.write("import cPickle, zlib\n\n\n")
            else:
                out.write("import cPickle\n\n\n")
        append = 1

        if compressed:
            out.write("def %sData():\n\treturn cPickle.loads(zlib.decompress(\n%s))\n\n"
                      % (varName, data))
        else:
            out.write("def %sData():\n\treturn cPickle.loads(\n%s)\n\n"
                      % (varName, data))
        if datatype == "Icon":
            out.write("def %sIcon():\n\tpm = QPixmap()\n\tpm.loadFromData(%sData())\n"
                      "\treturn pm\n\n"
                      "def %sIconSet():\n\treturn QIconSet(%sIcon())\n\n"
                      % tuple([varName] * 4))

        print "Embedded %s into %s using %s" % (fname, python_file, varName)


if __name__ == "__main__":
    main(sys.argv[1:])





